# Janos version 1.2.1
# JanosModel.py
# by Antoine Rauzy
# Copyrights (c) 2018-2022 AltaRica Association
#
# This file is part of Janos, a pedagogical stochastic simulator.
# Janos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Janos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Janos.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Methods to test types
# 4) Factory of elements
# 5) Methods for the parser
# 6) Methods for the checker

# 1) Imported modules
# -------------------

from S2MLCore import *

from JanosType import *
from JanosBlock import *
from JanosParameter import *
from JanosVariable import *
from JanosEquation import *
from JanosObserver import *

# 2) Creator
# ---------

class JanosModel(S2MLModel):

  def __init__(self):
    S2MLModel.__init__(self)

# 3) Methods to test types
# ------------------------

  @staticmethod
  def IsVariable(element):
    return element.GetType()==JanosType.VARIABLE

  @staticmethod
  def IsAssignment(element):
    return element.GetType()==JanosType.ASSIGNMENT

  @staticmethod
  def IsDoubleAssignment(element):
    return element.GetType()==JanosType.DOUBLE_ASSIGNMENT

  @staticmethod
  def IsConnection(element):
    return element.GetType()==JanosType.CONNECTION

  @staticmethod
  def IsObserver(element):
    return element.GetType()==JanosType.OBSERVER

# 4) Factory of elements
# ----------------------

  def NewBlock(self, path, container):
    return JanosBlock(path, container)

  def NewVariable(self, domain, path, container):
    return JanosVariable(domain, path, container)

  def NewParameter(self, domain, path, container):
    return JanosParameter(domain, path, container)

  def NewAssignment(self, leftMember, rightMember, container):
    return JanosEquation.NewAssignment(leftMember, rightMember, container)

  def NewDoubleAssignment(self, leftMember, rightMember, container):
    return JanosEquation.NewDoubleAssignment(leftMember, rightMember, container)

  def NewConnection(self, members, container):
    return JanosEquation.NewConnection(members, container)

  def NewObserver(self, domain, path, container):
    return JanosObserver(domain, path, container)

# 5) Methods for the parser
# -------------------------

  def CanComposeVariable(self, container):
    return True

  def CanComposeEquation(self, container):
    return True

  def CanComposeObserver(self, container):
    return True

# 6) Methods for the checker
# --------------------------

  def IsValidReferredElementInStochasticExpression(self, element):
    return element.GetType()==JanosType.PARAMETER or element.GetType()==JanosType.VARIABLE

