# Janos version 1.2.1
# JanosObserver.py
# by Antoine Rauzy
# Copyrights (c) 2018-2022 AltaRica Association
#
# This file is part of Janos, a pedagogical stochastic simulator.
# Janos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Janos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Janos.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Accessors

# 1) Imported modules
# -------------------

from S2MLCore import *

from JanosType import *

# 2) Observers
# ------------

class JanosObserver(S2MLPort):
  def __init__(self, domain, path, container):
    S2MLPort.__init__(self, JanosType.OBSERVER, domain, path, container)
    self._definition = None
    self._index = 0
    self._indicatorProfile = None
    self._indicators = []
    self._value = S2MLConstant.NewNone()

# 3) Accessors
# ------------

  def GetDefinition(self):
    return self._definition

  def SetDefinition(self, definition):
    self._definition = definition

  def GetIndex(self):
    return self._index

  def SetIndex(self, index):
    self._index = index

  def IsValueCalculated(self):
    return not self._value.IsNone()

  def ResetValue(self):
    self._value.Reset()

  def GetValue(self):
    return self._value

  def SetValue(self, value):
    self._value.Set(value)

  def GetIndicatorProfile(self):
    return self._indicatorProfile

  def SetIndicatorProfile(self, profile):
    self._indicatorProfile = profile

  def CreateIndicators(self, numberOfIndicators):
    domain = self.GetDomain()
    if domain.GetType()==S2MLDomain.REFERENCE:
      model = self.GetModel()
      declaration = model.LookForIdentifiedElement(domain.GetIdentifier())
      domain = declaration.GetDomain()
    self._indicators = [S2MLIndicator(self._indicatorProfile, domain) for i in range(0, numberOfIndicators)]

  def GetNumberOfIndicators(self):
    return len(self._indicators)

  def GetIndicators(self):
    return self._indicators

  def GetIndicator(self, index):
    if index<0 or index>=self.GetNumberOfIndicators():
      return None
    return self._indicators[index]

  def RecordValueInIndicator(self, index):
    if index<0 or index>=self.GetNumberOfIndicators():
      return
    indicator = self._indicators[index]
    indicator.AddValue(self._value)
